import * as i1 from '@angular/cdk/overlay';
import { OverlayConfig, OverlayModule } from '@angular/cdk/overlay';
import * as i2 from '@angular/cdk/portal';
import { BasePortalOutlet, CdkPortalOutlet, ComponentPortal, TemplatePortal, PortalModule } from '@angular/cdk/portal';
import * as i3 from '@angular/common';
import { DOCUMENT, CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { EventEmitter, Directive, ViewChild, Component, ViewEncapsulation, Injector, TemplateRef, Injectable, Input, Optional, Inject, Output, ContentChildren, NgModule } from '@angular/core';
import { Subject, take, merge, race } from 'rxjs';
import { filter, take as take$1, delay } from 'rxjs/operators';
import { trigger, state, style, transition, animate, query } from '@angular/animations';
import * as i2$1 from '@angular/cdk/a11y';

class FrontLayerConfig {
    constructor() {
        /** The ARIA role of the frontlayer element. */
        this.role = 'main';
        /** Aria label to assign to the frontlayer element. */
        this.ariaLabel = null;
        /** Data being injected into the child component. */
        this.data = null;
        /** Override for the dialog's top position. */
        this.top = '56px';
        /** Show the front-layer with elevation throwing a shadow on the underlying surface. */
        this.popover = false;
        /** Show the front-layer with transparent background. **/
        this.transparent = false;
        /** When the backlayer is revealed, the frontlayer content becomes inactive. */
        this.disableOnDrop = true;
        this.autoFocus = true;
        this.restoreFocus = true;
    }
    /** Merges the specified config with the default config */
    static merge(config) {
        let _config = new FrontLayerConfig();
        return { ..._config, ...config };
    }
}

class AnimationCurves {
}
AnimationCurves.STANDARD_CURVE = 'cubic-bezier(0.4,0.0,0.2,1)';
AnimationCurves.DECELERATION_CURVE = 'cubic-bezier(0.0,0.0,0.2,1)';
AnimationCurves.ACCELERATION_CURVE = 'cubic-bezier(0.4,0.0,1,1)';
AnimationCurves.SHARP_CURVE = 'cubic-bezier(0.4,0.0,0.6,1)';
/** @docs-private */
class AnimationDurations {
}
AnimationDurations.COMPLEX = '375ms';
AnimationDurations.ENTERING = '225ms';
AnimationDurations.EXITING = '195ms';
const BackdropAnimations = {
    frontLayerContainer: trigger('frontLayerContainer', [
        state('void, exit', style({ transform: 'translateY(100%)' })),
        state('enter', style({ transform: 'none' })),
        transition('* => enter', animate(`${AnimationDurations.EXITING} ${AnimationCurves.DECELERATION_CURVE}`, style({ transform: 'none' }))),
        transition('* => void, * => exit', animate(`${AnimationDurations.COMPLEX} ${AnimationCurves.ACCELERATION_CURVE}`)),
        transition('* => fading', query('@fade', [
            animate(`${AnimationDurations.EXITING} ${AnimationCurves.STANDARD_CURVE}`, style({ opacity: 1 }))
        ]))
    ]),
    frontLayerContainerOverlay: trigger('fade', []),
    backdropButton: trigger('rotate', [
        state('closed', style({ transform: 'none' })),
        state('opened', style({ transform: 'rotate(0.75turn)' })),
        transition('closed => opened', animate(`${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`)),
        transition('opened => closed', animate(`${AnimationDurations.EXITING} ${AnimationCurves.STANDARD_CURVE}`))
    ])
};

/**
 * Throws an exception for the case when a ComponentPortal is
 * attached to a DomPortalOutlet without an origin.
 * @docs-private
 */
function throwFrontLayerContentAlreadyAttachedError() {
    throw Error('Attempting to attach front-layer content after content is already attached');
}
function throwNoFrontLayerContentAttachedError() {
    throw Error('Attempting to detach front-layer content before any content has attached');
}
class _FrontLayerContainerBase extends BasePortalOutlet {
    constructor(
    /** The dialog configuration. */
    _config, _elementRef) {
        super();
        this._config = _config;
        this._elementRef = _elementRef;
        /** Emits when an animation state changes. */
        this._animationStateChanged = new EventEmitter();
    }
    /**
    * Attach a ComponentPortal as content to this front-layer container.
    * @param portal Portal to be attached as the front-layer content.
    */
    attachComponentPortal(portal) {
        if (this._portalOutlet.hasAttached()) {
            throwFrontLayerContentAlreadyAttachedError();
        }
        return this._portalOutlet.attachComponentPortal(portal);
    }
    /**
     * Attach a TemplatePortal as content to this front-layer container.
     * @param portal Portal to be attached as the front-layer content.
     */
    attachTemplatePortal(portal) {
        if (this._portalOutlet.hasAttached()) {
            throwFrontLayerContentAlreadyAttachedError();
        }
        return this._portalOutlet.attachTemplatePortal(portal);
    }
    detach() {
        if (!this._portalOutlet.hasAttached()) {
            throwNoFrontLayerContentAttachedError();
        }
        this._portalOutlet.detach();
    }
}
_FrontLayerContainerBase.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _FrontLayerContainerBase, deps: [{ token: FrontLayerConfig }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
_FrontLayerContainerBase.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: _FrontLayerContainerBase, viewQueries: [{ propertyName: "_portalOutlet", first: true, predicate: CdkPortalOutlet, descendants: true, static: true }], usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _FrontLayerContainerBase, decorators: [{
            type: Directive
        }], ctorParameters: function () { return [{ type: FrontLayerConfig }, { type: i0.ElementRef }]; }, propDecorators: { _portalOutlet: [{
                type: ViewChild,
                args: [CdkPortalOutlet, { static: true }]
            }] } });
class MatFrontLayerContainer extends _FrontLayerContainerBase {
    constructor() {
        super(...arguments);
        /** State of the dialog animation. */
        this._state = 'enter';
    }
    /** Callback, invoked whenever an animation on the host completes. */
    _onAnimationDone(event) {
        if (event.toState === 'enter') {
            this._animationStateChanged.next({ state: 'opened', totalTime: event.totalTime });
        }
        else if (event.toState === 'exit') {
            this._animationStateChanged.next({ state: 'closed', totalTime: event.totalTime });
        }
        else if (event.toState === 'fading') {
            this._animationStateChanged.next({ state: 'fading', totalTime: event.totalTime });
            this._state = 'enter';
        }
    }
    /** Callback, invoked when an animation on the host starts. */
    _onAnimationStart(event) {
        if (event.toState === 'enter') {
            this._animationStateChanged.next({ state: 'opening', totalTime: event.totalTime });
        }
        else if (event.toState === 'exit' || event.toState === 'void') {
            this._animationStateChanged.next({ state: 'closing', totalTime: event.totalTime });
        }
    }
    /** Starts the dialog exit animation. */
    _startExitAnimation() {
        this._state = 'exit';
    }
    _startDropAnimation() {
        this._state = 'droped';
    }
    _startFadingAnimation() {
        this._state = 'fading';
    }
    _startEnterAnimation() {
        this._state = 'enter';
    }
}
MatFrontLayerContainer.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontLayerContainer, deps: null, target: i0.ɵɵFactoryTarget.Component });
MatFrontLayerContainer.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontLayerContainer, selector: "mat-frontlayer-container", host: { attributes: { "tabindex": "-1" }, listeners: { "@frontLayerContainer.start": "_onAnimationStart($event)", "@frontLayerContainer.done": "_onAnimationDone($event)" }, properties: { "attr.role": "_config.role", "attr.aria-label": "_config.ariaLabel", "class.mat-frontlayer-container-popover": "_config.popover", "class.mat-frontlayer-container-transparency": "_config.transparent", "@frontLayerContainer": "_state" }, classAttribute: "mat-frontlayer-container" }, usesInheritance: true, ngImport: i0, template: "<ng-template cdkPortalOutlet></ng-template>\n<span @fade class=\"mat-frontlayer-content-fading-overlay\"></span>", styles: [".cdk-overlay-container,.cdk-global-overlay-wrapper{pointer-events:none;top:0;left:0;height:100%;width:100%}.cdk-overlay-container{position:fixed;z-index:1000}.cdk-overlay-container:empty{display:none}.cdk-global-overlay-wrapper{display:flex;position:absolute;z-index:1000}.cdk-overlay-pane{position:absolute;pointer-events:auto;box-sizing:border-box;z-index:1000;display:flex;max-width:100%;max-height:100%}.cdk-overlay-backdrop{position:absolute;top:0;bottom:0;left:0;right:0;z-index:1000;pointer-events:auto;-webkit-tap-highlight-color:transparent;transition:opacity .4s cubic-bezier(.25,.8,.25,1);opacity:0}.cdk-overlay-backdrop.cdk-overlay-backdrop-showing{opacity:1}.cdk-high-contrast-active .cdk-overlay-backdrop.cdk-overlay-backdrop-showing{opacity:.6}.cdk-overlay-dark-backdrop{background:rgba(0,0,0,.32)}.cdk-overlay-transparent-backdrop{transition:visibility 1ms linear,opacity 1ms linear;visibility:hidden;opacity:1}.cdk-overlay-transparent-backdrop.cdk-overlay-backdrop-showing{opacity:0;visibility:visible}.cdk-overlay-connected-position-bounding-box{position:absolute;z-index:1000;display:flex;flex-direction:column;min-width:1px;min-height:1px}.cdk-global-scrollblock{position:fixed;width:100%;overflow-y:scroll}.mat-frontlayer-container{background-color:#fff;padding-top:16px;padding-left:16px;padding-right:16px;border-top-left-radius:16px;border-top-right-radius:16px;width:100%;height:auto;display:flex;flex-direction:column;box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f;-webkit-box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f}.mat-frontlayer-container-popover{box-shadow:0 0 5px 2px #00000026}.mat-frontlayer-container-transparency{background-color:transparent}.mat-frontlayer-content-fading-overlay{background-color:inherit;opacity:0;position:absolute;z-index:1100;box-sizing:border-box;border-radius:inherit;width:100%;height:100%;top:inherit;left:0;pointer-events:none}@keyframes drop{0%{margin-top:var(--s, 56px)}to{margin-top:var(--e, 256px)}}@keyframes lift{0%{margin-top:var(--s, 256px)}to{margin-top:var(--e, 56px)}}@keyframes move{0%{margin-top:var(--s, 56px)}to{margin-top:var(--e, 256px)}}@keyframes move-horizontal{0%{transform:translate(var(--s, 0))}to{transform:translate(var(--e, -110%))}}.mat-frontlayer-content{margin:0 -16px;padding:0 16px 16px;display:block;flex:1;overflow-y:auto;-webkit-overflow-scrolling:touch}.mat-frontlayer-content-divider{border-top:1px solid #e0e0e0;position:sticky;top:0;height:0;z-index:2000}.mat-frontlayer-title{margin:0 0 16px;display:block}div.mat-frontlayer-title{display:flex;flex-direction:row;align-items:center}div.mat-frontlayer-title h1,div.mat-frontlayer-title h2,div.mat-frontlayer-title h3,div.mat-frontlayer-title h4,div.mat-frontlayer-title h5,div.mat-frontlayer-title h6{margin:0}.mat-frontlayer-actions{padding-top:8px;display:flex;flex-wrap:wrap;align-items:center;box-sizing:content-box;margin-bottom:8px}.mat-frontlayer-actions.mat-frontlayer-actions-align-center,.mat-frontlayer-actions[align=center]{justify-content:center}.mat-frontlayer-actions.mat-frontlayer-actions-align-end,.mat-frontlayer-actions[align=end]{justify-content:flex-end}.mat-frontlayer-actions .mat-button-base+.mat-button-base,.mat-frontlayer-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:8px}.mat-backlayer{width:100%;height:100%;display:flex;flex-direction:column}.mat-backlayer-title{flex:0 0 56px;padding:8px 5px;width:100%;display:flex;box-sizing:border-box;flex-direction:row;align-items:center;white-space:nowrap}.mat-backlayer-content{width:100%;padding-left:16px;padding-right:16px}.mat-backdrop-button-wrapper{width:40px;height:40px}.mat-backdrop-button{padding:0;width:40px;height:40px;line-height:40px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;overflow:hidden;outline:none;border-radius:50%;text-align:center;box-sizing:border-box;position:relative}.mat-backdrop-button .mat-icon{line-height:24px}.mat-backdrop-button.cdk-keyboard-focused .mat-backdrop-button-focus-overlay,.mat-backdrop-button.cdk-program-focused .mat-backdrop-button-focus-overlay{opacity:.12}.mat-backdrop-button:hover{cursor:pointer}.mat-backdrop-button .mat-backdrop-button-wrapper>*{vertical-align:middle}.mat-backdrop-button-focus-overlay{position:absolute;top:0;left:0;right:0;bottom:0;pointer-events:none;border-radius:inherit;background-color:#000;opacity:0;transition:opacity .2s cubic-bezier(.25,.8,.25,1),background-color .2s cubic-bezier(.25,.8,.25,1)}.mat-backlayer-move{padding:0;width:24px;height:24px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;overflow:hidden;outline:none}.mat-backlayer-move:hover{cursor:pointer}.mat-frontlayer-button{padding:0;width:24px;height:24px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;outline:none;border-radius:50%;position:relative}.mat-frontlayer-button.cdk-keyboard-focused .mat-frontlayer-button-focus-overlay,.mat-frontlayer-button.cdk-program-focused .mat-frontlayer-button-focus-overlay{opacity:.12}.mat-frontlayer-button:hover{cursor:pointer}.mat-frontlayer-button-focus-overlay{position:absolute;width:40px;height:40px;top:-8px;left:-8px;right:0;bottom:0;pointer-events:none;border-radius:inherit;background-color:#000;opacity:0;transition:opacity .2s cubic-bezier(.25,.8,.25,1),background-color .2s cubic-bezier(.25,.8,.25,1)}\n"], directives: [{ type: i2.CdkPortalOutlet, selector: "[cdkPortalOutlet]", inputs: ["cdkPortalOutlet"], outputs: ["attached"], exportAs: ["cdkPortalOutlet"] }], animations: [
        BackdropAnimations.frontLayerContainer,
        BackdropAnimations.frontLayerContainerOverlay
    ], encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontLayerContainer, decorators: [{
            type: Component,
            args: [{ selector: 'mat-frontlayer-container', encapsulation: ViewEncapsulation.None, animations: [
                        BackdropAnimations.frontLayerContainer,
                        BackdropAnimations.frontLayerContainerOverlay
                    ], host: {
                        'tabindex': '-1',
                        '[attr.role]': '_config.role',
                        '[attr.aria-label]': '_config.ariaLabel',
                        'class': 'mat-frontlayer-container',
                        '[class.mat-frontlayer-container-popover]': '_config.popover',
                        '[class.mat-frontlayer-container-transparency]': '_config.transparent',
                        '[@frontLayerContainer]': '_state',
                        '(@frontLayerContainer.start)': '_onAnimationStart($event)',
                        '(@frontLayerContainer.done)': '_onAnimationDone($event)'
                    }, template: "<ng-template cdkPortalOutlet></ng-template>\n<span @fade class=\"mat-frontlayer-content-fading-overlay\"></span>", styles: [".cdk-overlay-container,.cdk-global-overlay-wrapper{pointer-events:none;top:0;left:0;height:100%;width:100%}.cdk-overlay-container{position:fixed;z-index:1000}.cdk-overlay-container:empty{display:none}.cdk-global-overlay-wrapper{display:flex;position:absolute;z-index:1000}.cdk-overlay-pane{position:absolute;pointer-events:auto;box-sizing:border-box;z-index:1000;display:flex;max-width:100%;max-height:100%}.cdk-overlay-backdrop{position:absolute;top:0;bottom:0;left:0;right:0;z-index:1000;pointer-events:auto;-webkit-tap-highlight-color:transparent;transition:opacity .4s cubic-bezier(.25,.8,.25,1);opacity:0}.cdk-overlay-backdrop.cdk-overlay-backdrop-showing{opacity:1}.cdk-high-contrast-active .cdk-overlay-backdrop.cdk-overlay-backdrop-showing{opacity:.6}.cdk-overlay-dark-backdrop{background:rgba(0,0,0,.32)}.cdk-overlay-transparent-backdrop{transition:visibility 1ms linear,opacity 1ms linear;visibility:hidden;opacity:1}.cdk-overlay-transparent-backdrop.cdk-overlay-backdrop-showing{opacity:0;visibility:visible}.cdk-overlay-connected-position-bounding-box{position:absolute;z-index:1000;display:flex;flex-direction:column;min-width:1px;min-height:1px}.cdk-global-scrollblock{position:fixed;width:100%;overflow-y:scroll}.mat-frontlayer-container{background-color:#fff;padding-top:16px;padding-left:16px;padding-right:16px;border-top-left-radius:16px;border-top-right-radius:16px;width:100%;height:auto;display:flex;flex-direction:column;box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f;-webkit-box-shadow:0 2px 1px -1px #0003,0 1px 1px #00000024,0 1px 3px #0000001f}.mat-frontlayer-container-popover{box-shadow:0 0 5px 2px #00000026}.mat-frontlayer-container-transparency{background-color:transparent}.mat-frontlayer-content-fading-overlay{background-color:inherit;opacity:0;position:absolute;z-index:1100;box-sizing:border-box;border-radius:inherit;width:100%;height:100%;top:inherit;left:0;pointer-events:none}@keyframes drop{0%{margin-top:var(--s, 56px)}to{margin-top:var(--e, 256px)}}@keyframes lift{0%{margin-top:var(--s, 256px)}to{margin-top:var(--e, 56px)}}@keyframes move{0%{margin-top:var(--s, 56px)}to{margin-top:var(--e, 256px)}}@keyframes move-horizontal{0%{transform:translate(var(--s, 0))}to{transform:translate(var(--e, -110%))}}.mat-frontlayer-content{margin:0 -16px;padding:0 16px 16px;display:block;flex:1;overflow-y:auto;-webkit-overflow-scrolling:touch}.mat-frontlayer-content-divider{border-top:1px solid #e0e0e0;position:sticky;top:0;height:0;z-index:2000}.mat-frontlayer-title{margin:0 0 16px;display:block}div.mat-frontlayer-title{display:flex;flex-direction:row;align-items:center}div.mat-frontlayer-title h1,div.mat-frontlayer-title h2,div.mat-frontlayer-title h3,div.mat-frontlayer-title h4,div.mat-frontlayer-title h5,div.mat-frontlayer-title h6{margin:0}.mat-frontlayer-actions{padding-top:8px;display:flex;flex-wrap:wrap;align-items:center;box-sizing:content-box;margin-bottom:8px}.mat-frontlayer-actions.mat-frontlayer-actions-align-center,.mat-frontlayer-actions[align=center]{justify-content:center}.mat-frontlayer-actions.mat-frontlayer-actions-align-end,.mat-frontlayer-actions[align=end]{justify-content:flex-end}.mat-frontlayer-actions .mat-button-base+.mat-button-base,.mat-frontlayer-actions .mat-mdc-button-base+.mat-mdc-button-base{margin-left:8px}.mat-backlayer{width:100%;height:100%;display:flex;flex-direction:column}.mat-backlayer-title{flex:0 0 56px;padding:8px 5px;width:100%;display:flex;box-sizing:border-box;flex-direction:row;align-items:center;white-space:nowrap}.mat-backlayer-content{width:100%;padding-left:16px;padding-right:16px}.mat-backdrop-button-wrapper{width:40px;height:40px}.mat-backdrop-button{padding:0;width:40px;height:40px;line-height:40px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;overflow:hidden;outline:none;border-radius:50%;text-align:center;box-sizing:border-box;position:relative}.mat-backdrop-button .mat-icon{line-height:24px}.mat-backdrop-button.cdk-keyboard-focused .mat-backdrop-button-focus-overlay,.mat-backdrop-button.cdk-program-focused .mat-backdrop-button-focus-overlay{opacity:.12}.mat-backdrop-button:hover{cursor:pointer}.mat-backdrop-button .mat-backdrop-button-wrapper>*{vertical-align:middle}.mat-backdrop-button-focus-overlay{position:absolute;top:0;left:0;right:0;bottom:0;pointer-events:none;border-radius:inherit;background-color:#000;opacity:0;transition:opacity .2s cubic-bezier(.25,.8,.25,1),background-color .2s cubic-bezier(.25,.8,.25,1)}.mat-backlayer-move{padding:0;width:24px;height:24px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;overflow:hidden;outline:none}.mat-backlayer-move:hover{cursor:pointer}.mat-frontlayer-button{padding:0;width:24px;height:24px;color:unset;background-color:transparent;background-repeat:no-repeat;border:none;outline:none;border-radius:50%;position:relative}.mat-frontlayer-button.cdk-keyboard-focused .mat-frontlayer-button-focus-overlay,.mat-frontlayer-button.cdk-program-focused .mat-frontlayer-button-focus-overlay{opacity:.12}.mat-frontlayer-button:hover{cursor:pointer}.mat-frontlayer-button-focus-overlay{position:absolute;width:40px;height:40px;top:-8px;left:-8px;right:0;bottom:0;pointer-events:none;border-radius:inherit;background-color:#000;opacity:0;transition:opacity .2s cubic-bezier(.25,.8,.25,1),background-color .2s cubic-bezier(.25,.8,.25,1)}\n"] }]
        }] });

class FrontLayerGroupRef {
    constructor(_frontlayers, _active) {
        this._frontlayers = _frontlayers;
        this._active = _active;
        /** Subject for notifying that the active frontlayer of the group has finished droping down. */
        this._afterSwitch = new Subject();
        /** Subject for notifying the user that the frontlayer group has started closing. */
        this._beforeClose = new Subject();
    }
    selectedIndex() {
        return this._active;
    }
    switch(active) {
        if (this._frontlayers[this._active].getState() === 1 /* DROPED */) {
            this._frontlayers[this._active].afterLift().pipe(take(1)).subscribe(() => this._switch(active));
            this._frontlayers[this._active].lift();
        }
        else {
            this._switch(active);
        }
    }
    _switch(active) {
        if (active > this._active) {
            this._frontlayers[this._active].moveLeft();
            this._frontlayers[active].centerFromRight();
            this._active = active;
            this._afterSwitch.next(this._frontlayers[this._active]);
        }
        else if (active < this._active) {
            this._frontlayers[this._active].moveRight();
            this._frontlayers[active].centerFromLeft();
            this._active = active;
            this._afterSwitch.next(this._frontlayers[this._active]);
        }
    }
    /**
    * Gets an observable that is notified when the front-layer has finished lifting up.
    */
    afterSwitch() {
        return this._afterSwitch;
    }
    close() {
        this._beforeClose.next();
        this._frontlayers.forEach(frontlayer => frontlayer.close());
    }
    /**
    * Gets an observable that is notified when the frontlayer group starts closing.
    */
    beforeClose() {
        return this._beforeClose;
    }
}

// Counter for unique dialog ids.
let uniqueId = 0;
class FrontLayerRef {
    constructor(_overlayRef, _config, _containerInstance, id = `front-layer-${uniqueId++}`) {
        this._overlayRef = _overlayRef;
        this._config = _config;
        this._containerInstance = _containerInstance;
        this.id = id;
        /** Subject for notifying the user that the front-layer has finished closing. */
        this._afterClosed = new Subject();
        /** Subject for notifying that the front-layer has started droping down. */
        this._beforeDroped = new Subject();
        /** Subject for notifying that the front-layer has finished droping down. */
        this._afterDroped = new Subject();
        /** Subject for notifying that the front-layer has started lifting up. */
        this._beforeLift = new Subject();
        /** Subject for notifying that the front-layer has finished lifting up. */
        this._afterLift = new Subject();
        /** Current state of the front-layer. */
        this._state = 3 /* CLOSED */;
        /** This listener reacts on clicks on a droped front layer to lift it */
        this._clickEventListener = () => this.lift();
        _containerInstance._animationStateChanged.pipe(filter(event => event.state === 'opened'), take$1(1)).subscribe(() => {
            this._state = 0 /* OPEN */;
        });
        _overlayRef.detachments().subscribe(() => {
            this._afterClosed.next();
        });
        this._overlayWrapper = _overlayRef.hostElement;
    }
    moveLeft() {
        this._resetOverlayAnimation();
        this._overlayRef.overlayElement.style.setProperty('--s', '0');
        this._overlayRef.overlayElement.style.setProperty('--e', 'calc(-100% - 24px)');
        this._overlayRef.overlayElement.style.animation = `move-horizontal ${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.transform = 'translateX(calc(-100% - 24px))';
    }
    centerFromRight() {
        this._resetOverlayAnimation();
        this._overlayRef.overlayElement.style.setProperty('--s', 'calc(100% + 24px)');
        this._overlayRef.overlayElement.style.setProperty('--e', '0');
        this._overlayRef.overlayElement.style.animation = `move-horizontal ${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.transform = 'none';
    }
    centerFromLeft() {
        this._resetOverlayAnimation();
        this._overlayRef.overlayElement.style.setProperty('--s', 'calc(-100% - 24px)');
        this._overlayRef.overlayElement.style.setProperty('--e', '0');
        this._overlayRef.overlayElement.style.animation = `move-horizontal ${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.transform = 'none';
    }
    moveRight() {
        this._resetOverlayAnimation();
        this._overlayRef.overlayElement.style.setProperty('--s', '0');
        this._overlayRef.overlayElement.style.setProperty('--e', 'calc(100% + 24px)');
        this._overlayRef.overlayElement.style.animation = `move-horizontal ${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.transform = 'translateX(calc(100% + 24px))';
    }
    /**
     * Move the front-layer by the specified offset
     * @param offset The distance by which the plane is to be moved
     */
    drop(offset, autoFocus, restoreFocus) {
        this._config.autoFocus = autoFocus ? autoFocus : true;
        this._config.restoreFocus = restoreFocus ? restoreFocus : true;
        this._beforeDroped.next();
        if (this._config.disableOnDrop) {
            this.viewRef.rootNodes.forEach((el) => {
                if (el.style) {
                    el.style.opacity = '50%';
                    el.style.pointerEvents = 'none';
                }
            });
        }
        const top = this._containerInstance._config.top ? this._containerInstance._config.top : '0px';
        this._overlayRef.overlayElement.style.setProperty('--s', top);
        this._overlayRef.overlayElement.style.setProperty('--e', offset);
        this._overlayRef.overlayElement.style.animation = `drop ${AnimationDurations.ENTERING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.marginTop = offset;
        setTimeout(() => {
            this._state = 1 /* DROPED */;
            this._afterDroped.next();
            // lift frontlayer if user clicks on it
            this._overlayRef.overlayElement.addEventListener('click', this._clickEventListener);
        }, 225);
    }
    updatePosition(top) {
        this._resetOverlayAnimation();
        const _oldTop = this._overlayRef.overlayElement.style.marginTop ? this._overlayRef.overlayElement.style.marginTop : '0px';
        if (_oldTop < top) {
            this._overlayRef.overlayElement.style.height = `calc(100vh - ${top})`;
        }
        this._overlayRef.overlayElement.style.setProperty('--s', _oldTop);
        this._overlayRef.overlayElement.style.setProperty('--e', top);
        this._overlayRef.overlayElement.style.animation = `move ${AnimationDurations.EXITING} ${AnimationCurves.STANDARD_CURVE}`;
        setTimeout(() => {
            this._overlayRef.overlayElement.style.marginTop = top;
            this._overlayRef.overlayElement.style.height = `calc(100vh - ${top})`;
        }, 195);
    }
    _resetOverlayAnimation() {
        this._overlayRef.overlayElement.style.animation = 'none'; // remove last animation
        void this._overlayRef.overlayElement.offsetHeight; // trigger DOM reflow
        this._overlayRef.overlayElement.style.animation = null;
    }
    updateDropPosition(offset) {
        if (this._state == 1 /* DROPED */) {
            const top = this._overlayRef.overlayElement.style.marginTop ? this._overlayRef.overlayElement.style.marginTop : '0px';
            this._overlayRef.overlayElement.style.setProperty('--s', top);
            this._overlayRef.overlayElement.style.setProperty('--e', offset);
            this._overlayRef.overlayElement.style.animation = `move ${AnimationDurations.EXITING} ${AnimationCurves.STANDARD_CURVE}`;
            this._overlayRef.overlayElement.style.marginTop = offset;
            // setTimeout(() => {
            // }, 195);
        }
    }
    /**
     * Move the front-layer back to its original position
     */
    lift() {
        this._beforeLift.next();
        const top = this._containerInstance._config.top ? this._containerInstance._config.top : '0px';
        const offset = this._overlayRef.overlayElement.style.marginTop;
        this._overlayRef.overlayElement.style.setProperty('--s', offset);
        this._overlayRef.overlayElement.style.setProperty('--e', top);
        this._overlayRef.overlayElement.style.animation = `lift ${AnimationDurations.EXITING} ${AnimationCurves.STANDARD_CURVE}`;
        this._overlayRef.overlayElement.style.marginTop = top;
        this._overlayRef.overlayElement.removeEventListener('click', this._clickEventListener);
        setTimeout(() => {
            this.viewRef.rootNodes.forEach((el) => {
                if (el.style) {
                    el.style.opacity = '';
                    el.style.pointerEvents = '';
                }
            });
            this._state = 0 /* OPEN */;
            this._afterLift.next();
        }, 195);
    }
    /**
    * Close the front-layer.
    */
    close() {
        this._containerInstance._animationStateChanged.pipe(filter(event => event.state === 'closing'), take$1(1)).subscribe(event => {
            this._overlayRef.detachBackdrop();
            setTimeout(() => this._finishFrontLayerClose(), event.totalTime);
        });
        this._state = 2 /* CLOSING */;
        this._containerInstance._startExitAnimation();
    }
    _finishFrontLayerClose() {
        this._state = 3 /* CLOSED */;
        this._overlayRef.dispose();
    }
    /** Gets the current state of the dialog's lifecycle. */
    getState() {
        return this._state;
    }
    /**
    * Gets an observable that is notified when the front-layer is finished closing.
    */
    afterClosed() {
        return this._afterClosed;
    }
    /**
    * Gets an observable that is notified when the front-layer has started droping down.
    */
    beforeDroped() {
        return this._beforeDroped;
    }
    /**
    * Gets an observable that is notified when the front-layer has finished droping down.
    */
    afterDroped() {
        return this._afterDroped;
    }
    /**
    * Gets an observable that is notified when the front-layer has started lifting up.
    */
    beforeLift() {
        return this._beforeLift;
    }
    /**
    * Gets an observable that is notified when the front-layer has finished lifting up.
    */
    afterLift() {
        return this._afterLift;
    }
}

/**
 * Base class for backdrop services.
 */
class _BackdropBase {
    constructor(_overlay, _injector, _dialogContainerType) {
        this._overlay = _overlay;
        this._injector = _injector;
        this._dialogContainerType = _dialogContainerType;
        this._frontLayerRef = new Array();
        /** Subject for notifying the user that the frontlayer has finished opening. */
        this._afterOpened = new Subject();
        /** Subject for notifying the user that the frontlayer has started closing. */
        this._beforeClosed = new Subject();
        /** Subject for notifiying the user that the content of the frontlayer has been replaced. */
        this._afterContentChanged = new Subject();
        /** Subject for notifiying the user that the content of the frontlayer has been replaced. */
        this._afterTabChanged = new Subject();
    }
    /**
     * Gets an observable that is notified when the front-layer is finished opening.
     */
    afterOpened() {
        return this._afterOpened;
    }
    /**
     * Gets an observable that is notified when the front-layer has started closing.
     */
    beforeClosed() {
        return this._beforeClosed;
    }
    /**
     * Gets an observable that is notified when the content the frontlayer has been changed.
     */
    afterContentChanged() {
        return this._afterContentChanged;
    }
    /**
     * Gets an observable that is notified when the content the frontlayer has been changed.
     */
    afterTabChanged() {
        return this._afterTabChanged;
    }
    /**
    * Opens a frontlayer containing the given component type or template. If there is already a frontlayer opened,
    * its content will get replaced with the given component type or template.
    * @param componentOrTemplateRef ComponentType or TemplateRef to instantiate as the frontlayer content.
    * @param config Configuration
    * @returns Reference to the newly-opened frontlayer.
    */
    open(componentOrTemplateRef, config) {
        // merge config or create default config
        let _config = config ? FrontLayerConfig.merge(config) : new FrontLayerConfig();
        if (this._frontLayerRef.length > 0 && _config.popover === false) {
            this._getClosestFrontlayer()._containerInstance._animationStateChanged.pipe(filter(event => event.state === 'fading')).subscribe(() => {
                this._getClosestFrontlayer()._containerInstance.detach();
                this._attachFrontLayerContent(componentOrTemplateRef, this._getClosestFrontlayer());
                this._getClosestFrontlayer()._config = _config;
                this._getClosestFrontlayer().updatePosition(_config.top);
                this._afterContentChanged.next();
            });
            this._getClosestFrontlayer()._containerInstance._startFadingAnimation();
            return this._getClosestFrontlayer();
        }
        const overlayRef = this._createOverlay(_config);
        const frontLayerContainer = this._attachFrontLayerContainer(overlayRef, _config);
        const animationStateSubscription = frontLayerContainer._animationStateChanged.pipe(filter(event => event.state === 'opened' || event.state === 'closing')).subscribe(frontLayerAnimationEvent => {
            if (frontLayerAnimationEvent.state === 'opened') {
                this._afterOpened.next();
            }
            else if (frontLayerAnimationEvent.state === 'closing') {
                if (this._getClosestFrontlayer()._config.popover === false) {
                    this._beforeClosed.next();
                }
                animationStateSubscription.unsubscribe();
            }
        });
        let _frontLayerRef = this._createFrontlayer(componentOrTemplateRef, frontLayerContainer, overlayRef, _config);
        this._frontLayerRef.push(_frontLayerRef);
        _frontLayerRef.afterClosed().subscribe(() => this._frontLayerRef.pop());
        return _frontLayerRef;
    }
    openGroup(templateRefs, active, config) {
        console.warn('Caution: You are using an EXPERIMENTAL feature of ngx-mat-backdrop.');
        let _frontlayers = new Array();
        let _config = config ? FrontLayerConfig.merge(config) : new FrontLayerConfig();
        templateRefs.forEach((templateRef, index) => {
            let overlayRef = this._createOverlay(_config);
            if (index < active) {
                overlayRef.overlayElement.style.transform = 'translateX(calc(-100% - 24px))';
            }
            else if (index > active) {
                overlayRef.overlayElement.style.transform = 'translateX(calc(100% + 24px))';
            }
            const frontLayerContainer = this._attachFrontLayerContainer(overlayRef, _config);
            if (index === active) {
                const animationStateSubscription = frontLayerContainer._animationStateChanged.pipe(filter(event => event.state === 'opened' || event.state === 'closing')).subscribe(frontLayerAnimationEvent => {
                    if (frontLayerAnimationEvent.state === 'opened') {
                        this._afterOpened.next();
                    }
                    else if (frontLayerAnimationEvent.state === 'closing') {
                        if (this._getClosestFrontlayer()._config.popover === false) {
                            this._beforeClosed.next();
                        }
                        animationStateSubscription.unsubscribe();
                    }
                });
            }
            let _frontLayerRef = this._createFrontlayer(templateRef, frontLayerContainer, overlayRef, _config);
            _frontlayers.push(_frontLayerRef);
            if (index === active) {
                this._frontLayerRef.push(_frontLayerRef);
            }
        });
        let _frontLayerGroup = new FrontLayerGroupRef(_frontlayers, active);
        _frontLayerGroup.afterSwitch().subscribe(_frontLayer => {
            this._frontLayerRef.pop();
            this._frontLayerRef.push(_frontLayer);
            this._afterTabChanged.next();
        });
        this._frontLayerGroupRef = _frontLayerGroup;
        this._frontLayerGroupRef.beforeClose().subscribe(() => _frontLayerGroup = null);
        return _frontLayerGroup;
    }
    /**
     * Finds the closest FrontLayerRef by looking at the internal stack of layers.
     * @returns A FrontLayerRef
     */
    _getClosestFrontlayer() {
        return this._frontLayerRef[this._frontLayerRef.length - 1];
    }
    /**
     * Creates the overlay into which the front-layer will be loaded.
     * @returns A promise resolving to the OverlayRef for the created overlay.
     */
    _createOverlay(config) {
        const overlayConfig = this._getOverlayConfig(config);
        return this._overlay.create(overlayConfig);
    }
    /**
     * Creates an default overlay config.
     * @returns The overlay configuration.
     */
    _getOverlayConfig(frontLayerConfig) {
        let top = frontLayerConfig.popover ? `calc(${this.getOpenedFrontLayer()?._config.top} + ${frontLayerConfig.top})` : frontLayerConfig.top;
        let _config = new OverlayConfig({
            width: '100%',
            height: 'calc(100vh - ' + top + ')',
            positionStrategy: this._overlay.position().global().top(top),
            scrollStrategy: this._overlay.scrollStrategies.block()
        });
        if (frontLayerConfig.popover) {
            _config.hasBackdrop = true;
            _config.backdropClass = 'cdk-overlay-transparent-backdrop';
        }
        return _config;
    }
    /**
    * Attaches a front-layer container to a dialog's already-created overlay.
    * @param overlay Reference to the dialog's underlying overlay.
    * @returns A promise resolving to a ComponentRef for the attached container.
    */
    _attachFrontLayerContainer(overlay, config) {
        const userInjector = config && config.viewContainerRef && config.viewContainerRef.injector;
        const injector = Injector.create({
            parent: userInjector || this._injector,
            providers: [{ provide: FrontLayerConfig, useValue: config }]
        });
        const containerPortal = new ComponentPortal(this._dialogContainerType, config.viewContainerRef, injector, config.componentFactoryResolver);
        const containerRef = overlay.attach(containerPortal);
        return containerRef.instance;
    }
    /**
    * Creates a new Frontlayer containing the given component type or template.
    * @param componentOrTemplateRef The type of component being loaded into the frontlayer,
    *     or a TemplateRef to instantiate as the content.
    * @param frontLayerContainer Reference to the wrapping frontlayer container.
    * @param overlayRef Reference to the overlay in which the front-layer resides.
    * @returns A FrontLayerRef that should be returned to the user.
    */
    _createFrontlayer(componentOrTemplateRef, frontLayerContainer, overlayRef, config) {
        let frontLayerRef = new FrontLayerRef(overlayRef, config, frontLayerContainer, config.id);
        this._attachFrontLayerContent(componentOrTemplateRef, frontLayerRef);
        return frontLayerRef;
    }
    /**
     * Attaches a user-provided component to the specified frontlayer.
     * @param componentOrTemplateRef The type of component being loaded into the frontlayer,
    *     or a TemplateRef to instantiate as the content.
     * @param frontLayerRef Reference to the wrapping frontlayer.
     */
    _attachFrontLayerContent(componentOrTemplateRef, frontLayerRef) {
        if (componentOrTemplateRef instanceof TemplateRef) {
            const viewRef = frontLayerRef._containerInstance.attachTemplatePortal(new TemplatePortal(componentOrTemplateRef, null, { $implicit: frontLayerRef._config.data, frontLayerRef }));
            frontLayerRef.viewRef = viewRef;
        }
        else {
            // TODO: Testen und ViewContainerRef holen und Datenübergabe mit Injector - Frage: Wo kommt die Default Config her?
            // const injector = this._createInjector<T>(config, frontLayerRef, frontLayerContainer);
            const contentRef = frontLayerRef._containerInstance.attachComponentPortal(new ComponentPortal(componentOrTemplateRef, frontLayerRef._config.viewContainerRef, null));
            frontLayerRef.componentInstance = contentRef.instance;
            frontLayerRef.viewRef = contentRef.hostView;
        }
    }
    /**
     * Finds the current opened front-layer.
     */
    getOpenedFrontLayer() {
        return this._getClosestFrontlayer();
    }
    /**
     * Finds the current opend frontlayer group
     */
    getOpenFrontLayerGroup() {
        return this._frontLayerGroupRef;
    }
}
_BackdropBase.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _BackdropBase, deps: [{ token: i1.Overlay }, { token: i0.Injector }, { token: i0.Type }], target: i0.ɵɵFactoryTarget.Directive });
_BackdropBase.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: _BackdropBase, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _BackdropBase, decorators: [{
            type: Directive
        }], ctorParameters: function () { return [{ type: i1.Overlay }, { type: i0.Injector }, { type: i0.Type }]; } });
class Backdrop extends _BackdropBase {
    constructor(overlay, injector) {
        super(overlay, injector, MatFrontLayerContainer);
    }
}
Backdrop.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: Backdrop, deps: [{ token: i1.Overlay }, { token: i0.Injector }], target: i0.ɵɵFactoryTarget.Injectable });
Backdrop.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: Backdrop });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: Backdrop, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i1.Overlay }, { type: i0.Injector }]; } });

class MatFrontlayer {
}
MatFrontlayer.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayer, deps: [], target: i0.ɵɵFactoryTarget.Component });
MatFrontlayer.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayer, selector: "mat-frontlayer", inputs: { name: "name", topPosition: "topPosition" }, viewQueries: [{ propertyName: "templateRef", first: true, predicate: TemplateRef, descendants: true }], ngImport: i0, template: `<ng-template><ng-content></ng-content></ng-template>`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayer, decorators: [{
            type: Component,
            args: [{
                    selector: 'mat-frontlayer',
                    template: `<ng-template><ng-content></ng-content></ng-template>`
                }]
        }], propDecorators: { templateRef: [{
                type: ViewChild,
                args: [TemplateRef]
            }], name: [{
                type: Input
            }], topPosition: [{
                type: Input
            }] } });
class MatBackdrop {
}
MatBackdrop.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdrop, deps: [], target: i0.ɵɵFactoryTarget.Component });
MatBackdrop.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBackdrop, selector: "mat-backdrop", ngImport: i0, template: `<ng-content select="mat-backlayer"></ng-content>`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdrop, decorators: [{
            type: Component,
            args: [{
                    selector: 'mat-backdrop',
                    template: `<ng-content select="mat-backlayer"></ng-content>`,
                }]
        }] });
class MatBackdropTrigger {
    constructor(_backdrop) {
        this._backdrop = _backdrop;
        this._frontLayer = null;
    }
    /** References the front layer instance that the trigger is associated with. */
    get frontLayer() {
        return this._frontLayer;
    }
    set frontLayer(frontLayer) {
        this._frontLayer = frontLayer;
    }
    ngAfterViewInit() {
        // if (this._frontLayer instanceof MatFrontlayerGroup) {
        //     let _config: FrontLayerConfig = new FrontLayerConfig();
        //     let _frontLayerRefs = this._frontLayer._allTabs.map(element => element.templateRef);
        //     this._backdrop.openGroup(_frontLayerRefs, this._frontLayer.active, _config);
        // } else {
        // if (this._frontLayer instanceof MatFrontlayer) {
        if (this._frontLayer?.templateRef) {
            let _config = new FrontLayerConfig();
            if (this._frontLayer.name) {
                _config.id = this._frontLayer.name;
            }
            if (this._frontLayer.topPosition) {
                _config.top = this._frontLayer.topPosition;
            }
            this._backdrop.open(this._frontLayer.templateRef, _config);
        }
        // }
    }
}
MatBackdropTrigger.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropTrigger, deps: [{ token: Backdrop }], target: i0.ɵɵFactoryTarget.Directive });
MatBackdropTrigger.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatBackdropTrigger, selector: "mat-backdrop[matBackdropTriggerFor], mat-backdrop[mat-backdrop-trigger-for]", inputs: { frontLayer: ["matBackdropTriggerFor", "frontLayer"] }, exportAs: ["matBackdropTriggerFor"], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropTrigger, decorators: [{
            type: Directive,
            args: [{
                    selector: `mat-backdrop[matBackdropTriggerFor], mat-backdrop[mat-backdrop-trigger-for]`,
                    exportAs: 'matBackdropTriggerFor'
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }]; }, propDecorators: { frontLayer: [{
                type: Input,
                args: ['matBackdropTriggerFor']
            }] } });
class MatBacklayer {
    constructor(_document, _backdrop, _element, _focusMonitor, _focusTrapFactory, _liveAnnouncer) {
        // force refreshing UI in a seperate thread with delay() to avoid 'Expression has changed after it was checked' error
        // merge(this._backdrop.afterOpened(), this._backdrop.afterContentChanged())
        //     .pipe(delay(0), take(1))
        //     .subscribe(() => this.showContextMenu = true);
        this._backdrop = _backdrop;
        this._element = _element;
        this._focusMonitor = _focusMonitor;
        this._focusTrapFactory = _focusTrapFactory;
        this._liveAnnouncer = _liveAnnouncer;
        this.showContextMenu = false;
        /** Element that was focused before the backlayer was revealed. Save this to restore upon close. */
        // private _elementFocusedBeforeDialogWasOpened: HTMLElement | null = null;
        /**
       * Type of interaction that led to the dialog being closed. This is used to determine
       * whether the focus style will be applied when returning focus to its original location
       * after the dialog is closed.
       */
        this._closeInteractionType = null;
        // this._backdrop.beforeClosed()
        //     .pipe(take(1))
        //     .subscribe(() => this.showContextMenu = false);
        this._document = _document;
        merge(this._backdrop.afterOpened(), this._backdrop.afterContentChanged(), this._backdrop.afterTabChanged())
            .pipe(delay(0))
            .subscribe(() => {
            let _frontlayer = this._backdrop.getOpenedFrontLayer();
            _frontlayer?.beforeDroped().subscribe(() => {
                this._liveAnnouncer.announce('Context menu revealed');
                this.showContextMenu = true;
                // FIXME: you can'ct rely on setTImeout, there is no guarantee the DOM will be rendered when the setTImeout callback is called
                // See: https://stackoverflow.com/questions/50743396/get-children-of-parent-element-after-ngif-angular-5
                setTimeout(() => this._trapFocus(_frontlayer));
            });
            _frontlayer?.afterLift().subscribe(() => {
                this._liveAnnouncer.announce('Context menu concealed');
                this.showContextMenu = false;
                setTimeout(() => this._restoreFocus(_frontlayer));
            });
        });
    }
    _trapFocus(_frontlayer) {
        this._focusTrap = this._focusTrapFactory.create(this._element.nativeElement);
        // Save the previously focused element. This element will be re-focused when the backlayer conceales
        // if (this._document) {
        //     this._elementFocusedBeforeDialogWasOpened = _getFocusedElementPierceShadowDom();
        // }
        let _autoFocus = _frontlayer._config.autoFocus;
        if (typeof _autoFocus === 'string') {
            this._focusByCssSelector(_autoFocus);
        }
        else if (_autoFocus === true) {
            this._focusTrap.focusInitialElementWhenReady().then(success => {
                if (!success) {
                    this._element.nativeElement.focus();
                }
            });
        }
    }
    _restoreFocus(_frontlayer) {
        this._focusTrap.destroy();
        let _restoreFocus = _frontlayer._config.restoreFocus;
        let _focusTargetElement;
        if (typeof _restoreFocus === 'string') {
            _focusTargetElement = _frontlayer._containerInstance._elementRef.nativeElement
                .querySelector(_restoreFocus);
        }
        else if (_restoreFocus === true) {
            _focusTargetElement = _frontlayer._containerInstance._elementRef.nativeElement
                .querySelector('h1, h2, h3, h4, h5, h6, [role="heading"]');
            if (_focusTargetElement == null) {
                _focusTargetElement = this._getFirstTabbable(_frontlayer);
            }
        }
        if (_focusTargetElement == null) {
            _focusTargetElement = _frontlayer._containerInstance._elementRef.nativeElement;
        }
        this._focusMonitor.focusVia(_focusTargetElement, this._closeInteractionType);
        this._closeInteractionType = null;
    }
    _getFirstTabbable(_frontlayer) {
        let _tabbables = _frontlayer._containerInstance._elementRef.nativeElement
            .querySelectorAll('button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])');
        if (_tabbables && _tabbables.length > 0) {
            return _tabbables[0];
        }
        else {
            return null;
        }
    }
    /**
   * Focuses the first element that matches the given selector within the focus trap.
   * @param selector The CSS selector for the element to set focus to.
   */
    _focusByCssSelector(selector, options) {
        let elementToFocus = this._element.nativeElement.querySelector(selector);
        if (elementToFocus == null) {
            elementToFocus = this._element.nativeElement;
        }
        elementToFocus?.focus();
    }
    ngOnInit() {
        // // force refreshing UI in a seperate thread with delay() to avoid 'Expression has changed after it was checked' error
        // this._afterOpenedSubscription = this._backdrop.afterOpened().pipe(delay(0))
        //     .subscribe(() => this.showContextMenu = true);
        // this._beforeClosedSubscription = this._backdrop.beforeClosed()
        //     .subscribe(() => this.showContextMenu = false);
    }
}
MatBacklayer.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayer, deps: [{ token: DOCUMENT, optional: true }, { token: Backdrop }, { token: i0.ElementRef }, { token: i2$1.FocusMonitor }, { token: i2$1.FocusTrapFactory }, { token: i2$1.LiveAnnouncer }], target: i0.ɵɵFactoryTarget.Component });
MatBacklayer.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayer, selector: "mat-backlayer", inputs: { color: "color" }, host: { properties: { "class.mat-backlayer-primary": "color === \"primary\"", "class.mat-backlayer-accent": "color === \"accent\"" }, classAttribute: "mat-backlayer" }, ngImport: i0, template: `
        <ng-content select="mat-backlayer-title"></ng-content>
        <ng-content select="mat-backlayer-content" *ngIf="showContextMenu"></ng-content>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayer, decorators: [{
            type: Component,
            args: [{
                    selector: 'mat-backlayer',
                    template: `
        <ng-content select="mat-backlayer-title"></ng-content>
        <ng-content select="mat-backlayer-content" *ngIf="showContextMenu"></ng-content>
    `,
                    inputs: ['color'],
                    host: {
                        'class': 'mat-backlayer',
                        '[class.mat-backlayer-primary]': 'color === "primary"',
                        '[class.mat-backlayer-accent]': 'color === "accent"'
                    }
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: Backdrop }, { type: i0.ElementRef }, { type: i2$1.FocusMonitor }, { type: i2$1.FocusTrapFactory }, { type: i2$1.LiveAnnouncer }]; } });
class MatBacklayerTitle {
}
MatBacklayerTitle.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerTitle, deps: [], target: i0.ɵɵFactoryTarget.Directive });
MatBacklayerTitle.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerTitle, selector: "mat-backlayer-title, [mat-backlayer-title], [matBacklayerTitle]", host: { properties: { "attr.role": "\"navigation\"" }, classAttribute: "mat-backlayer-title" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerTitle, decorators: [{
            type: Directive,
            args: [{
                    selector: 'mat-backlayer-title, [mat-backlayer-title], [matBacklayerTitle]',
                    host: {
                        'class': 'mat-backlayer-title',
                        '[attr.role]': '"navigation"'
                    }
                }]
        }] });
class MatBacklayerContent {
}
MatBacklayerContent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerContent, deps: [], target: i0.ɵɵFactoryTarget.Directive });
MatBacklayerContent.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerContent, selector: "mat-backlayer-content, [mat-backlayer-content], [matBacklayerContent]", host: { properties: { "attr.role": "\"menu\"", "attr.aria-label": "\"context\"" }, classAttribute: "mat-backlayer-content" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerContent, decorators: [{
            type: Directive,
            args: [{
                    selector: 'mat-backlayer-content, [mat-backlayer-content], [matBacklayerContent]',
                    host: {
                        'class': 'mat-backlayer-content',
                        '[attr.role]': '"menu"',
                        '[attr.aria-label]': '"context"'
                    }
                }]
        }] });
class _MatBacklayerButton {
    constructor(_elementRef, _focusMonitor) {
        this._elementRef = _elementRef;
        this._focusMonitor = _focusMonitor;
    }
    ngAfterViewInit() {
        this._focusMonitor.monitor(this._elementRef, true);
    }
    ngOnDestroy() {
        this._focusMonitor.stopMonitoring(this._elementRef);
    }
    focus(origin, options) {
        if (origin) {
            this._focusMonitor.focusVia(this._getHostElement(), origin, options);
        }
        else {
            this._getHostElement().focus(options);
        }
    }
    _getHostElement() {
        return this._elementRef.nativeElement;
    }
}
_MatBacklayerButton.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _MatBacklayerButton, deps: [{ token: i0.ElementRef }, { token: i2$1.FocusMonitor }], target: i0.ɵɵFactoryTarget.Directive });
_MatBacklayerButton.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: _MatBacklayerButton, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: _MatBacklayerButton, decorators: [{
            type: Directive,
            args: [{}]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i2$1.FocusMonitor }]; } });
class MatBacklayerToggle extends _MatBacklayerButton {
    constructor(_backdrop, elementRef, focusMonitor) {
        super(elementRef, focusMonitor);
        this._backdrop = _backdrop;
        this.elementRef = elementRef;
        this.focusMonitor = focusMonitor;
        this._arialabel = 'Reveal context menu';
        this.offset = '200px';
        this.open = new EventEmitter();
        this.close = new EventEmitter();
        this._state = 'void';
    }
    ngOnInit() {
        this._backdrop.afterOpened()
            .pipe(take$1(1))
            .subscribe(() => this._init());
        this._backdrop.afterTabChanged().subscribe(() => this._init());
        this._backdrop.afterContentChanged()
            .pipe(take$1(1))
            .subscribe(() => this._init());
    }
    _init() {
        this._frontLayerRef = this._backdrop.getOpenedFrontLayer();
        this._frontLayerRef?.beforeDroped().subscribe(() => {
            this._state = 'opened';
        });
        this._frontLayerRef?.afterDroped().subscribe(() => {
            this._arialabel = 'Conceal context menu';
            this.open.emit();
        });
        this._frontLayerRef?.beforeLift().subscribe(() => {
            this._state = 'void';
        });
        this._frontLayerRef?.afterLift().subscribe(() => {
            this._arialabel = 'Reveal context menu';
            this.close.emit();
        });
        if (this.offset === 'full') {
            this.offset = 'calc(100vh - 56px)';
        }
    }
    _getFrontLayer() {
        return this._frontLayerRef;
    }
    _opened() {
        if (this._frontLayerRef && this._frontLayerRef.getState() === 1 /* DROPED */) {
            return true;
        }
        else {
            return false;
        }
    }
    _onClick() {
        if (this._state === 'void') {
            this._frontLayerRef?.drop(this.offset, this.autoFocus, this.restoreFocus);
        }
        else {
            this._frontLayerRef?.lift();
        }
    }
}
MatBacklayerToggle.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerToggle, deps: [{ token: Backdrop }, { token: i0.ElementRef }, { token: i2$1.FocusMonitor }], target: i0.ɵɵFactoryTarget.Component });
MatBacklayerToggle.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerToggle, selector: "button[mat-backlayer-toggle], button[matBacklayerToggle]", inputs: { offset: "offset", autoFocus: "autoFocus", restoreFocus: "restoreFocus" }, outputs: { open: "open", close: "close" }, host: { listeners: { "click": "_onClick()" }, properties: { "attr.aria-label": "_arialabel" }, classAttribute: "mat-backdrop-button" }, usesInheritance: true, ngImport: i0, template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], animations: [BackdropAnimations.backdropButton] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerToggle, decorators: [{
            type: Component,
            args: [{
                    selector: 'button[mat-backlayer-toggle], button[matBacklayerToggle]',
                    animations: [BackdropAnimations.backdropButton],
                    template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `,
                    host: {
                        'class': 'mat-backdrop-button',
                        '(click)': '_onClick()',
                        '[attr.aria-label]': '_arialabel'
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }, { type: i0.ElementRef }, { type: i2$1.FocusMonitor }]; }, propDecorators: { offset: [{
                type: Input
            }], autoFocus: [{
                type: Input
            }], restoreFocus: [{
                type: Input
            }], open: [{
                type: Output
            }], close: [{
                type: Output
            }] } });
class MatBacklayerClose extends _MatBacklayerButton {
    constructor(_backdrop, elementRef, focusMonitor) {
        super(elementRef, focusMonitor);
        this._backdrop = _backdrop;
        this.elementRef = elementRef;
        this.focusMonitor = focusMonitor;
        this.close = new EventEmitter();
        this.default = new EventEmitter();
        this._state = 'void';
    }
    ngOnInit() {
        // Set tabindex so anchor link is accessable with tab
        let _originTabIndex = this.elementRef.nativeElement.getAttribute('tabindex');
        if (_originTabIndex == null || _originTabIndex === '') {
            this._originAriaLabel = this.elementRef.nativeElement.setAttribute('tabindex', '0');
        }
        race(this._backdrop.afterOpened(), this._backdrop.afterContentChanged())
            .pipe(take$1(1))
            .subscribe(() => {
            this._init();
        });
    }
    _init() {
        this._frontLayerRef = this._backdrop.getOpenedFrontLayer();
        this._beforeDropedSubscription = this._frontLayerRef?.beforeDroped().subscribe(() => {
            this._state = 'opened';
            this._originAriaLabel = this.elementRef.nativeElement.getAttribute('aria-label');
            this.elementRef.nativeElement.setAttribute('aria-label', 'Conceal Context Menu');
        });
        this._beforeLiftSubscription = this._frontLayerRef?.beforeLift().subscribe(() => {
            this._state = 'void';
            this.elementRef.nativeElement.setAttribute('aria-label', this._originAriaLabel ? this._originAriaLabel : '');
        });
        this._afterLiftSubscription = this._frontLayerRef?.afterLift().subscribe(() => {
            this.close.emit();
        });
    }
    ngOnDestroy() {
        super.ngOnDestroy();
        this._beforeDropedSubscription?.unsubscribe();
        this._beforeLiftSubscription?.unsubscribe();
        this._afterLiftSubscription?.unsubscribe();
    }
    _opened() {
        if (this._frontLayerRef && this._frontLayerRef.getState() === 1 /* DROPED */) {
            return true;
        }
        else {
            return false;
        }
    }
    _onClick() {
        if (this._state === 'opened') {
            this._frontLayerRef?.lift();
        }
        else {
            this.default.emit();
        }
    }
}
MatBacklayerClose.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerClose, deps: [{ token: Backdrop }, { token: i0.ElementRef }, { token: i2$1.FocusMonitor }], target: i0.ɵɵFactoryTarget.Component });
MatBacklayerClose.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerClose, selector: "button[mat-backlayer-close], button[matBacklayerClose]", outputs: { close: "close", default: "default" }, host: { listeners: { "keydown.enter": "$event.preventDefault() ; $event.target.click() ;", "keydown.space": "$event.preventDefault() ; $event.target.click() ;", "click": "_onClick()" }, classAttribute: "mat-backdrop-button" }, usesInheritance: true, ngImport: i0, template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], animations: [BackdropAnimations.backdropButton] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerClose, decorators: [{
            type: Component,
            args: [{
                    selector: 'button[mat-backlayer-close], button[matBacklayerClose]',
                    animations: [BackdropAnimations.backdropButton],
                    template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `,
                    host: {
                        'class': 'mat-backdrop-button',
                        "(keydown.enter)": "$event.preventDefault() ; $event.target.click() ;",
                        "(keydown.space)": "$event.preventDefault() ; $event.target.click() ;",
                        '(click)': '_onClick()',
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }, { type: i0.ElementRef }, { type: i2$1.FocusMonitor }]; }, propDecorators: { close: [{
                type: Output
            }], default: [{
                type: Output
            }] } });
class MatBacklayerCloseAnchor extends MatBacklayerClose {
    constructor() {
        super(...arguments);
        this._role = 'link';
    }
    _init() {
        super._init();
        let _frontLayerRef = this._backdrop.getOpenedFrontLayer();
        _frontLayerRef?.beforeDroped().subscribe(() => this._role = 'button');
        _frontLayerRef?.beforeLift().subscribe(() => this._role = 'link');
    }
}
MatBacklayerCloseAnchor.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerCloseAnchor, deps: null, target: i0.ɵɵFactoryTarget.Component });
MatBacklayerCloseAnchor.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerCloseAnchor, selector: "a[mat-backlayer-close], a[matBacklayerClose]", host: { properties: { "attr.role": "_role" } }, usesInheritance: true, ngImport: i0, template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], animations: [BackdropAnimations.backdropButton] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerCloseAnchor, decorators: [{
            type: Component,
            args: [{
                    selector: 'a[mat-backlayer-close], a[matBacklayerClose]',
                    animations: [BackdropAnimations.backdropButton],
                    template: `
        <ng-container *ngIf="!_opened(); else close">
            <div class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <ng-content></ng-content>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-container>
        <ng-template #close>
            <div aria-hidden="true" class="mat-backdrop-button-wrapper" [@rotate]="_state === 'opened' ? 'opened' : 'closed'">
                <svg xmlns="http://www.w3.org/2000/svg" height="24px" viewBox="0 0 24 24" width="24px" fill="currentColor">
                    <path d="M0 0h24v24H0z" fill="none" />
                    <path
                        d="M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z" />
                </svg>
            </div>
            <span class="mat-backdrop-button-focus-overlay"></span>
        </ng-template>
    `,
                    host: {
                        '[attr.role]': '_role'
                    }
                }]
        }] });
class MatBacklayerMove extends _MatBacklayerButton {
    constructor(_backdrop, elementRef, focusMonitor) {
        super(elementRef, focusMonitor);
        this._backdrop = _backdrop;
        this.elementRef = elementRef;
        this.focusMonitor = focusMonitor;
        this.offset = '200px';
        this.move = new EventEmitter();
    }
    ngOnInit() {
        if (this.offset === 'full') {
            this.offset = 'calc(100vh - 56px)';
        }
    }
    _onClick() {
        let _frontLayerRef = this._backdrop.getOpenedFrontLayer();
        if (_frontLayerRef) {
            if (_frontLayerRef.getState() === 1 /* DROPED */) {
                _frontLayerRef.updateDropPosition(this.offset);
            }
            else {
                _frontLayerRef.drop(this.offset, this.autoFocus, this.restoreFocus);
            }
            this.move.emit();
        }
    }
}
MatBacklayerMove.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerMove, deps: [{ token: Backdrop }, { token: i0.ElementRef }, { token: i2$1.FocusMonitor }], target: i0.ɵɵFactoryTarget.Component });
MatBacklayerMove.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerMove, selector: "button[mat-backlayer-move], button[matBacklayerMove]", inputs: { offset: "offset", autoFocus: "autoFocus", restoreFocus: "restoreFocus" }, outputs: { move: "move" }, host: { listeners: { "click": "_onClick()" }, classAttribute: "mat-backdrop-button" }, usesInheritance: true, ngImport: i0, template: `
        <span class="mat-backdrop-button-wrapper"><ng-content></ng-content></span>
        <span class="mat-backdrop-button-focus-overlay"></span>
    `, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerMove, decorators: [{
            type: Component,
            args: [{
                    selector: `button[mat-backlayer-move], button[matBacklayerMove]`,
                    template: `
        <span class="mat-backdrop-button-wrapper"><ng-content></ng-content></span>
        <span class="mat-backdrop-button-focus-overlay"></span>
    `,
                    host: {
                        'class': 'mat-backdrop-button',
                        '(click)': '_onClick()'
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }, { type: i0.ElementRef }, { type: i2$1.FocusMonitor }]; }, propDecorators: { offset: [{
                type: Input
            }], autoFocus: [{
                type: Input
            }], restoreFocus: [{
                type: Input
            }], move: [{
                type: Output
            }] } });
class MatBacklayerSwitchTab {
    constructor(_backdrop) {
        this._backdrop = _backdrop;
    }
    set active(active) {
        this._active = active;
    }
    _onClick() {
        this._backdrop.getOpenFrontLayerGroup()?.switch(this._active);
    }
}
MatBacklayerSwitchTab.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerSwitchTab, deps: [{ token: Backdrop }], target: i0.ɵɵFactoryTarget.Directive });
MatBacklayerSwitchTab.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatBacklayerSwitchTab, selector: "button[mat-backlayer-switch-tab], button[matBacklayerSwitchTab]", inputs: { active: ["mat-backlayer-switch-tab", "active"] }, host: { listeners: { "click": "_onClick()" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBacklayerSwitchTab, decorators: [{
            type: Directive,
            args: [{
                    selector: 'button[mat-backlayer-switch-tab], button[matBacklayerSwitchTab]',
                    host: {
                        '(click)': '_onClick()'
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }]; }, propDecorators: { active: [{
                type: Input,
                args: ['mat-backlayer-switch-tab']
            }] } });

/**
 * Scrollable content container of a front-layer.
 */
class MatFrontlayerContent {
    constructor() {
        this.showDividerOnScroll = true;
        this._scrolling = false;
    }
    _onScroll(event) {
        this._scrolling = event.srcElement.scrollTop > 0;
    }
}
MatFrontlayerContent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerContent, deps: [], target: i0.ɵɵFactoryTarget.Component });
MatFrontlayerContent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayerContent, selector: "[mat-frontlayer-content], mat-frontlayer-content, [matFrontLayerContent]", inputs: { showDividerOnScroll: "showDividerOnScroll" }, host: { listeners: { "scroll": "_onScroll($event)" }, properties: { "attr.role": "\"region\"", "attr.aria-label": "\"content\"" }, classAttribute: "mat-frontlayer-content" }, ngImport: i0, template: `
        <div class="mat-frontlayer-content-divider" *ngIf="showDividerOnScroll && _scrolling"></div>
        <ng-content></ng-content>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerContent, decorators: [{
            type: Component,
            args: [{
                    selector: `[mat-frontlayer-content], mat-frontlayer-content, [matFrontLayerContent]`,
                    template: `
        <div class="mat-frontlayer-content-divider" *ngIf="showDividerOnScroll && _scrolling"></div>
        <ng-content></ng-content>
    `,
                    host: {
                        'class': 'mat-frontlayer-content',
                        '(scroll)': '_onScroll($event)',
                        '[attr.role]': '"region"',
                        '[attr.aria-label]': '"content"'
                    }
                }]
        }], propDecorators: { showDividerOnScroll: [{
                type: Input
            }] } });
/**
 * Title of a front-layer element. Stays fixed to the top of the dialog when scrolling.
 */
class MatFrontlayerTitle {
}
MatFrontlayerTitle.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerTitle, deps: [], target: i0.ɵɵFactoryTarget.Directive });
MatFrontlayerTitle.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayerTitle, selector: "[mat-frontlayer-title], [matFrontLayerTitle]", host: { classAttribute: "mat-frontlayer-title" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerTitle, decorators: [{
            type: Directive,
            args: [{
                    selector: `[mat-frontlayer-title], [matFrontLayerTitle]`,
                    host: {
                        'class': 'mat-frontlayer-title'
                    }
                }]
        }] });
class MatFrontlayerActions {
    constructor() {
        /**
        * Horizontal alignment of action buttons.
        */
        this.align = 'start';
    }
}
MatFrontlayerActions.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerActions, deps: [], target: i0.ɵɵFactoryTarget.Directive });
MatFrontlayerActions.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayerActions, selector: "[mat-frontlayer-actions], mat-frontlayer-actions, [matFrontlayerActions]", inputs: { align: "align" }, host: { properties: { "class.mat-frontlayer-actions-align-center": "align === \"center\"", "class.mat-frontlayer-actions-align-end": "align === \"end\"", "attr.role": "\"footer\"", "attr.aria-label": "\"content actions\"" }, classAttribute: "mat-frontlayer-actions" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerActions, decorators: [{
            type: Directive,
            args: [{
                    selector: `[mat-frontlayer-actions], mat-frontlayer-actions, [matFrontlayerActions]`,
                    host: {
                        'class': 'mat-frontlayer-actions',
                        '[class.mat-frontlayer-actions-align-center]': 'align === "center"',
                        '[class.mat-frontlayer-actions-align-end]': 'align === "end"',
                        '[attr.role]': '"footer"',
                        '[attr.aria-label]': '"content actions"'
                    }
                }]
        }], propDecorators: { align: [{
                type: Input
            }] } });
class MatFrontlayerDrop extends _MatBacklayerButton {
    constructor(_backdrop, elementRef, focusMonitor) {
        super(elementRef, focusMonitor);
        this._backdrop = _backdrop;
        this.elementRef = elementRef;
        this.focusMonitor = focusMonitor;
        this.offset = '200px';
        this.drop = new EventEmitter();
        this._show = true;
    }
    ngOnInit() {
        this._backdrop.afterOpened().pipe(take(1)).subscribe(() => {
            this._init();
        });
        this._backdrop.afterContentChanged().pipe(take(1)).subscribe(() => {
            this._init();
        });
    }
    _init() {
        let _frontlayer = this._backdrop.getOpenedFrontLayer();
        if (_frontlayer) {
            _frontlayer.beforeDroped().subscribe(() => this._show = false);
            _frontlayer.afterLift().subscribe(() => this._show = true);
        }
    }
    _onClick() {
        let _frontLayerRef = this._backdrop.getOpenedFrontLayer();
        if (_frontLayerRef) {
            _frontLayerRef.drop(this.offset, this.autoFocus, this.restoreFocus);
            this.drop.emit();
        }
    }
}
MatFrontlayerDrop.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerDrop, deps: [{ token: Backdrop }, { token: i0.ElementRef }, { token: i2$1.FocusMonitor }], target: i0.ɵɵFactoryTarget.Component });
MatFrontlayerDrop.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayerDrop, selector: "button[mat-frontlayer-drop], button[matFrontlayerDrop]", inputs: { offset: "offset", autoFocus: "autoFocus", restoreFocus: "restoreFocus" }, outputs: { drop: "drop" }, host: { listeners: { "click": "_onClick()" }, classAttribute: "mat-frontlayer-button" }, usesInheritance: true, ngImport: i0, template: `
        <ng-container *ngIf="_show">
            <span class="mat-backdrop-button-wrapper"><ng-content></ng-content></span>
            <span class="mat-frontlayer-button-focus-overlay"></span>
        </ng-container>
    `, isInline: true, directives: [{ type: i3.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerDrop, decorators: [{
            type: Component,
            args: [{
                    selector: 'button[mat-frontlayer-drop], button[matFrontlayerDrop]',
                    template: `
        <ng-container *ngIf="_show">
            <span class="mat-backdrop-button-wrapper"><ng-content></ng-content></span>
            <span class="mat-frontlayer-button-focus-overlay"></span>
        </ng-container>
    `,
                    host: {
                        'class': 'mat-frontlayer-button',
                        '(click)': '_onClick()'
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }, { type: i0.ElementRef }, { type: i2$1.FocusMonitor }]; }, propDecorators: { offset: [{
                type: Input
            }], autoFocus: [{
                type: Input
            }], restoreFocus: [{
                type: Input
            }], drop: [{
                type: Output
            }] } });
class MatFrontLayerClose {
    constructor(_backdrop) {
        this._backdrop = _backdrop;
    }
    _onClick() {
        let _frontLayerRef = this._backdrop.getOpenedFrontLayer();
        if (_frontLayerRef) {
            _frontLayerRef.close();
        }
    }
}
MatFrontLayerClose.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontLayerClose, deps: [{ token: Backdrop }], target: i0.ɵɵFactoryTarget.Directive });
MatFrontLayerClose.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontLayerClose, selector: "button[mat-frontlayer-close], button[matFrontlayerClose]", host: { listeners: { "click": "_onClick()" }, properties: { "attr.aria-label": "\"Conceal context menu\"" }, classAttribute: "mat-frontlayer-button" }, exportAs: ["matFrontlayerClose"], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontLayerClose, decorators: [{
            type: Directive,
            args: [{
                    selector: 'button[mat-frontlayer-close], button[matFrontlayerClose]',
                    exportAs: 'matFrontlayerClose',
                    host: {
                        'class': 'mat-frontlayer-button',
                        '(click)': '_onClick()',
                        '[attr.aria-label]': '"Conceal context menu"'
                    }
                }]
        }], ctorParameters: function () { return [{ type: Backdrop }]; } });
class MatFrontlayerGroup {
    constructor() {
        this.active = 0;
    }
}
MatFrontlayerGroup.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerGroup, deps: [], target: i0.ɵɵFactoryTarget.Component });
MatFrontlayerGroup.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.11", type: MatFrontlayerGroup, selector: "mat-frontlayer-group", inputs: { active: "active" }, queries: [{ propertyName: "_allTabs", predicate: MatFrontlayer, descendants: true }], ngImport: i0, template: `<ng-content></ng-content>`, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatFrontlayerGroup, decorators: [{
            type: Component,
            args: [{
                    selector: 'mat-frontlayer-group',
                    template: `<ng-content></ng-content>`
                }]
        }], propDecorators: { _allTabs: [{
                type: ContentChildren,
                args: [MatFrontlayer, { descendants: true }]
            }], active: [{
                type: Input
            }] } });

class MatBackdropModule {
    static forRoot() {
        return {
            ngModule: MatBackdropModule,
            providers: [Backdrop]
        };
    }
}
MatBackdropModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
MatBackdropModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropModule, declarations: [MatBackdrop,
        MatBackdropTrigger,
        MatBacklayer,
        MatBacklayerTitle,
        MatBacklayerToggle,
        MatBacklayerClose,
        MatBacklayerCloseAnchor,
        MatBacklayerMove,
        MatBacklayerSwitchTab,
        MatBacklayerContent,
        MatFrontlayer,
        MatFrontlayerGroup,
        MatFrontLayerContainer,
        MatFrontlayerContent,
        MatFrontlayerTitle,
        MatFrontlayerActions,
        MatFrontlayerDrop,
        MatFrontLayerClose], imports: [CommonModule,
        OverlayModule,
        PortalModule], exports: [MatBacklayer,
        MatBackdropTrigger,
        MatBacklayerTitle,
        MatBacklayerToggle,
        MatBacklayerClose,
        MatBacklayerCloseAnchor,
        MatBacklayerMove,
        MatBacklayerSwitchTab,
        MatBacklayerContent,
        MatFrontlayerContent,
        MatFrontlayerTitle,
        MatFrontlayerActions,
        MatBackdrop,
        MatFrontlayer,
        MatFrontlayerGroup,
        MatFrontlayerDrop,
        MatFrontLayerClose] });
MatBackdropModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropModule, imports: [[
            CommonModule,
            OverlayModule,
            PortalModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.11", ngImport: i0, type: MatBackdropModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        MatBackdrop,
                        MatBackdropTrigger,
                        MatBacklayer,
                        MatBacklayerTitle,
                        MatBacklayerToggle,
                        MatBacklayerClose,
                        MatBacklayerCloseAnchor,
                        MatBacklayerMove,
                        MatBacklayerSwitchTab,
                        MatBacklayerContent,
                        MatFrontlayer,
                        MatFrontlayerGroup,
                        MatFrontLayerContainer,
                        MatFrontlayerContent,
                        MatFrontlayerTitle,
                        MatFrontlayerActions,
                        MatFrontlayerDrop,
                        MatFrontLayerClose
                    ],
                    imports: [
                        CommonModule,
                        OverlayModule,
                        PortalModule
                    ],
                    exports: [
                        MatBacklayer,
                        MatBackdropTrigger,
                        MatBacklayerTitle,
                        MatBacklayerToggle,
                        MatBacklayerClose,
                        MatBacklayerCloseAnchor,
                        MatBacklayerMove,
                        MatBacklayerSwitchTab,
                        MatBacklayerContent,
                        MatFrontlayerContent,
                        MatFrontlayerTitle,
                        MatFrontlayerActions,
                        MatBackdrop,
                        MatFrontlayer,
                        MatFrontlayerGroup,
                        MatFrontlayerDrop,
                        MatFrontLayerClose
                    ]
                }]
        }] });

/*
 * Public API Surface of ngx-mat-backdrop
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Backdrop, BackdropAnimations, FrontLayerConfig, FrontLayerGroupRef, FrontLayerRef, MatBackdrop, MatBackdropModule, MatBackdropTrigger, MatBacklayer, MatBacklayerClose, MatBacklayerCloseAnchor, MatBacklayerContent, MatBacklayerMove, MatBacklayerSwitchTab, MatBacklayerTitle, MatBacklayerToggle, MatFrontLayerClose, MatFrontlayer, MatFrontlayerActions, MatFrontlayerContent, MatFrontlayerDrop, MatFrontlayerGroup, MatFrontlayerTitle, _BackdropBase, _MatBacklayerButton };
//# sourceMappingURL=ngx-mat-backdrop.mjs.map
