import { ComponentType, Overlay } from '@angular/cdk/overlay';
import { Injector, TemplateRef, Type } from '@angular/core';
import { Observable } from 'rxjs';
import { FrontLayerConfig } from './front-layer-config';
import { MatFrontLayerContainer, _FrontLayerContainerBase } from './front-layer-container';
import { FrontLayerGroupRef } from './front-layer-group-ref';
import { FrontLayerRef } from './front-layer-ref';
import * as i0 from "@angular/core";
/**
 * Base class for backdrop services.
 */
export declare abstract class _BackdropBase<C extends _FrontLayerContainerBase> {
    private _overlay;
    private _injector;
    private _dialogContainerType;
    private _frontLayerRef;
    private _frontLayerGroupRef;
    /** Subject for notifying the user that the frontlayer has finished opening. */
    private readonly _afterOpened;
    /** Subject for notifying the user that the frontlayer has started closing. */
    private readonly _beforeClosed;
    /** Subject for notifiying the user that the content of the frontlayer has been replaced. */
    private readonly _afterContentChanged;
    /** Subject for notifiying the user that the content of the frontlayer has been replaced. */
    private readonly _afterTabChanged;
    /**
     * Gets an observable that is notified when the front-layer is finished opening.
     */
    afterOpened(): Observable<void>;
    /**
     * Gets an observable that is notified when the front-layer has started closing.
     */
    beforeClosed(): Observable<void>;
    /**
     * Gets an observable that is notified when the content the frontlayer has been changed.
     */
    afterContentChanged(): Observable<void>;
    /**
     * Gets an observable that is notified when the content the frontlayer has been changed.
     */
    afterTabChanged(): Observable<void>;
    constructor(_overlay: Overlay, _injector: Injector, _dialogContainerType: Type<C>);
    /**
    * Opens a frontlayer containing the given component type or template. If there is already a frontlayer opened,
    * its content will get replaced with the given component type or template.
    * @param componentOrTemplateRef ComponentType or TemplateRef to instantiate as the frontlayer content.
    * @param config Configuration
    * @returns Reference to the newly-opened frontlayer.
    */
    open<T, D = any>(componentOrTemplateRef: ComponentType<T> | TemplateRef<T>, config?: FrontLayerConfig<D>): FrontLayerRef<T>;
    openGroup<D = any>(templateRefs: TemplateRef<any>[], active: number, config?: FrontLayerConfig<D>): FrontLayerGroupRef;
    /**
     * Finds the closest FrontLayerRef by looking at the internal stack of layers.
     * @returns A FrontLayerRef
     */
    private _getClosestFrontlayer;
    /**
     * Creates the overlay into which the front-layer will be loaded.
     * @returns A promise resolving to the OverlayRef for the created overlay.
     */
    private _createOverlay;
    /**
     * Creates an default overlay config.
     * @returns The overlay configuration.
     */
    private _getOverlayConfig;
    /**
    * Attaches a front-layer container to a dialog's already-created overlay.
    * @param overlay Reference to the dialog's underlying overlay.
    * @returns A promise resolving to a ComponentRef for the attached container.
    */
    private _attachFrontLayerContainer;
    /**
    * Creates a new Frontlayer containing the given component type or template.
    * @param componentOrTemplateRef The type of component being loaded into the frontlayer,
    *     or a TemplateRef to instantiate as the content.
    * @param frontLayerContainer Reference to the wrapping frontlayer container.
    * @param overlayRef Reference to the overlay in which the front-layer resides.
    * @returns A FrontLayerRef that should be returned to the user.
    */
    private _createFrontlayer;
    /**
     * Attaches a user-provided component to the specified frontlayer.
     * @param componentOrTemplateRef The type of component being loaded into the frontlayer,
    *     or a TemplateRef to instantiate as the content.
     * @param frontLayerRef Reference to the wrapping frontlayer.
     */
    private _attachFrontLayerContent;
    /**
     * Finds the current opened front-layer.
     */
    getOpenedFrontLayer(): FrontLayerRef<any> | undefined;
    /**
     * Finds the current opend frontlayer group
     */
    getOpenFrontLayerGroup(): FrontLayerGroupRef;
    static ɵfac: i0.ɵɵFactoryDeclaration<_BackdropBase<any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<_BackdropBase<any>, never, never, {}, {}, never>;
}
export declare class Backdrop extends _BackdropBase<MatFrontLayerContainer> {
    constructor(overlay: Overlay, injector: Injector);
    static ɵfac: i0.ɵɵFactoryDeclaration<Backdrop, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<Backdrop>;
}
