import { BasePortalOutlet, CdkPortalOutlet, ComponentPortal, TemplatePortal } from '@angular/cdk/portal';
import { ComponentRef, ElementRef, EmbeddedViewRef, EventEmitter } from '@angular/core';
import { AnimationEvent } from '@angular/animations';
import { FrontLayerConfig } from './front-layer-config';
import * as i0 from "@angular/core";
/** Event that captures the state of dialog container animations. */
interface FrontLayerAnimationEvent {
    state: 'opened' | 'opening' | 'closing' | 'closed' | 'fading';
    totalTime: number;
}
/**
 * Throws an exception for the case when a ComponentPortal is
 * attached to a DomPortalOutlet without an origin.
 * @docs-private
 */
export declare function throwFrontLayerContentAlreadyAttachedError(): void;
export declare function throwNoFrontLayerContentAttachedError(): void;
export declare abstract class _FrontLayerContainerBase extends BasePortalOutlet {
    /** The dialog configuration. */
    _config: FrontLayerConfig;
    _elementRef: ElementRef;
    /** Starts the front-layer exit animation */
    abstract _startExitAnimation(): void;
    /** Starts the front-layer drop animation  **/
    abstract _startDropAnimation(): void;
    /** Starts the front-layer fading animation (fade out)  **/
    abstract _startFadingAnimation(): void;
    /** Starts the frontlayer enter animation */
    abstract _startEnterAnimation(): void;
    /** Emits when an animation state changes. */
    _animationStateChanged: EventEmitter<FrontLayerAnimationEvent>;
    /** The portal outlet inside of this container into which the dialog content will be loaded. */
    _portalOutlet: CdkPortalOutlet;
    constructor(
    /** The dialog configuration. */
    _config: FrontLayerConfig, _elementRef: ElementRef);
    /**
    * Attach a ComponentPortal as content to this front-layer container.
    * @param portal Portal to be attached as the front-layer content.
    */
    attachComponentPortal<T>(portal: ComponentPortal<T>): ComponentRef<T>;
    /**
     * Attach a TemplatePortal as content to this front-layer container.
     * @param portal Portal to be attached as the front-layer content.
     */
    attachTemplatePortal<C>(portal: TemplatePortal<C>): EmbeddedViewRef<C>;
    detach(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<_FrontLayerContainerBase, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<_FrontLayerContainerBase, never, never, {}, {}, never>;
}
export declare class MatFrontLayerContainer extends _FrontLayerContainerBase {
    /** State of the dialog animation. */
    _state: 'void' | 'enter' | 'droped' | 'fading' | 'exit';
    /** Callback, invoked whenever an animation on the host completes. */
    _onAnimationDone(event: AnimationEvent): void;
    /** Callback, invoked when an animation on the host starts. */
    _onAnimationStart(event: AnimationEvent): void;
    /** Starts the dialog exit animation. */
    _startExitAnimation(): void;
    _startDropAnimation(): void;
    _startFadingAnimation(): void;
    _startEnterAnimation(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<MatFrontLayerContainer, never>;
    static ɵcmp: i0.ɵɵComponentDeclaration<MatFrontLayerContainer, "mat-frontlayer-container", never, {}, {}, never, never>;
}
export {};
