import { OverlayRef } from "@angular/cdk/overlay";
import { EmbeddedViewRef } from "@angular/core";
import { Observable } from "rxjs";
import { FrontLayerConfig } from "./front-layer-config";
import { _FrontLayerContainerBase } from "./front-layer-container";
/** Possible states of the lifecycle of a front-layer. */
export declare const enum FrontLayerState {
    OPEN = 0,
    DROPED = 1,
    CLOSING = 2,
    CLOSED = 3
}
export declare class FrontLayerRef<T> {
    private _overlayRef;
    _config: FrontLayerConfig;
    _containerInstance: _FrontLayerContainerBase;
    readonly id: string;
    /** The instance of component opened into the front-layer. */
    componentInstance: T;
    /** A viewRef of the component opened into the front-layer  */
    viewRef: EmbeddedViewRef<T>;
    /** Subject for notifying the user that the front-layer has finished closing. */
    private readonly _afterClosed;
    /** Subject for notifying that the front-layer has started droping down. */
    private readonly _beforeDroped;
    /** Subject for notifying that the front-layer has finished droping down. */
    private readonly _afterDroped;
    /** Subject for notifying that the front-layer has started lifting up. */
    private readonly _beforeLift;
    /** Subject for notifying that the front-layer has finished lifting up. */
    private readonly _afterLift;
    /** Current state of the front-layer. */
    private _state;
    /** CDK Overlay Wrapper element of the front-layer */
    private _overlayWrapper;
    /** This listener reacts on clicks on a droped front layer to lift it */
    private _clickEventListener;
    constructor(_overlayRef: OverlayRef, _config: FrontLayerConfig, _containerInstance: _FrontLayerContainerBase, id?: string);
    moveLeft(): void;
    centerFromRight(): void;
    centerFromLeft(): void;
    moveRight(): void;
    /**
     * Move the front-layer by the specified offset
     * @param offset The distance by which the plane is to be moved
     */
    drop(offset: string, autoFocus?: string | boolean, restoreFocus?: string | boolean): void;
    updatePosition(top: string): void;
    private _resetOverlayAnimation;
    updateDropPosition(offset: string): void;
    /**
     * Move the front-layer back to its original position
     */
    lift(): void;
    /**
    * Close the front-layer.
    */
    close(): void;
    private _finishFrontLayerClose;
    /** Gets the current state of the dialog's lifecycle. */
    getState(): FrontLayerState;
    /**
    * Gets an observable that is notified when the front-layer is finished closing.
    */
    afterClosed(): Observable<void>;
    /**
    * Gets an observable that is notified when the front-layer has started droping down.
    */
    beforeDroped(): Observable<void>;
    /**
    * Gets an observable that is notified when the front-layer has finished droping down.
    */
    afterDroped(): Observable<void>;
    /**
    * Gets an observable that is notified when the front-layer has started lifting up.
    */
    beforeLift(): Observable<void>;
    /**
    * Gets an observable that is notified when the front-layer has finished lifting up.
    */
    afterLift(): Observable<void>;
}
